#include "libhash.h"

/* By Paul Hsieh (C) 2004.  Covered under the Paul Hsieh
   derivative license. For license details see:

   http://www.azillionmonkeys.com/qed/weblicense.html

   http://www.azillionmonkeys.com/qed/hash.html */

#undef getshort
#if (defined(__GNUC__) && defined(__i386__)) || defined(__WATCOMC__)
# define getshort(d) (*((const unsigned short *) (d)))
#endif
#if defined(_MSC_VER) || defined(__BORLANDC__)
# define getshort(d) (*((const unsigned short *) (d)))
#endif

#if !defined(getshort)
# define getshort(d) ((((const unsigned char *) (d))[1] << 8UL)\
                      +((const unsigned char *) (d))[0])
#endif

/* ------------------ */

/* Note by cbf - The portability of this is suspect.
   It does seem to be well suited for aligned binary
   items (was SuperFastHash) */
unsigned long sfh_ph(const char *data, size_t len)
{
   unsigned long   hash;
   int             i, rem;

   if (len <= 0 || data == NULL) return 0;

   hash = (unsigned long)len;   /* Avoid 0 -> 0 trap */
   rem = len & 3;
   len >>= 2;

   /* Main loop */
   for (i = 0; i < len; i++) {
      hash += getshort(data);
      data += sizeof(short);
      hash ^= hash << 16;
      hash ^= getshort(data) << 11;
      data += sizeof(short);
      hash += hash >> 11;
   }

   /* Handle end cases */
   switch (rem) {
case 3:
      hash += getshort(data);
      hash ^= hash << 16;
      hash ^= data[sizeof(short)] << 18;
      hash += hash >> 11;
      break;
case 2:
      hash += getshort(data);
      hash ^= hash << 11;
      hash += hash >> 17;
      break;
case 1:
      hash += *data;
      hash ^= hash << 10;
      hash += hash >> 1;
   }

   /* Force "avalanching" of final 127 bits */
   hash ^= hash << 3;
   hash += hash >> 5;
   hash ^= hash << 2;
   hash += hash >> 15;
   hash ^= hash << 10;

   return hash;
} /* sfh_ph */
